@@ -19,6 +19,8 @@ use PDL::IO::FastRaw;
 use PDL::CCS;
 use PDL::CCS::IO::FastRaw;
 use Fcntl qw(:DEFAULT SEEK_SET SEEK_CUR SEEK_END);
+use File::Basename qw(dirname);
+use version;
 use strict;
 
 ##==============================================================================
@@ -47,6 +49,7 @@ BEGIN {
 ##   ##-- logging options
 ##   logvprofile => $level, ##-- log-level for vprofile() (default=undef:none)
 ##   logio => $level,       ##-- log-level for low-level I/O operations (default=undef:none)
+##   logCompat => $level,   ##-- log-level for compatibility warnings (default='warn')
 ##   ##
 ##   ##-- modelling options (formerly via DocClassify)
 ##   minFreq    => $fmin,   ##-- minimum total term-frequency for model inclusion (default=undef:use $coldb->{tfmin})
@@ -60,6 +63,7 @@ BEGIN {
 ##   ##-- guts: aux: info
 ##   N => $tdm0Total,       ##-- total number of (doc,term) frequencies counted
 ##   dbreak => $dbreak,     ##-- inherited from $coldb on create()
+##   version => $version,   ##-- file version, for compatibility checks
 ##   ##
 ##   ##-- guts: aux: term-tuples ($NA:number of term-attributes, $NT:number of term-tuples)
 ##   attrs  => \@attrs,       ##-- known term attributes
@@ -78,6 +82,8 @@ BEGIN {
 ##   tdm => $tdm,             ##-- term-doc matrix : PDL::CCS::Nd ($NT,$ND): [$ti,$di] -> f($ti,$di)
 ##   tym => $tym,             ##-- term-year matrix: PDL::CCS::Nd ($NT,$NY): [$ti,$yi] -> f($ti,$yi)
 ##   cf  => $cf_pdl,          ##-- cat-freq pdl:     dense:       ($NC)    : [$ci]     -> f($ci)
+##   yf  => $yf_pdl,          ##-- year-freq pdl:    dense:       ($NY)    : [$yi-$y0] -> f($yi)
+##   y0  => $y0,              ##-- minimum year: scalar
 ##   #tf  => $tf_pdl,          ##-- term-freq pdl:   dense:       ($NT)    : [$ti]     -> f($ti)
 ##   #tw  => $tw_pdl,          ##-- term-weight pdl: dense:       ($NT)    : [$ti]     -> ($wRaw=0) + ($wCooked=1)*w($ti)
 ##   #                         ##   + where w($t) = 1 - H(Doc|T=$t) / H_max(Doc) ~ DocClassify termWeight=>'max-entropy-quotient'
@@ -108,7 +114,10 @@ sub new {
 			       attrs => [],
 			       ##
 			       logvprofile  => 'trace',
-			       logio => 'trace',
+			       logio => undef, #'trace',
+			       logCompat => 'warn',
+			       ##
+			       version => $DiaColloDB::VERSION,
 			       ##
 			       @_
 			      );
@@ -193,9 +202,23 @@ sub open {
   $vs->{base}  = $base;
   $vs->{flags} = $flags = fcflags($flags);
 
+  my ($hdr); ##-- save header, for version-checking
   if (fcread($flags) && !fctrunc($flags)) {
-    $vs->loadHeader()
-      or $vs->logconess("failed to load header from '$vs->{base}.hdr': $!");
+    $hdr = $vs->readHeader()
+      or $vs->logconess("failed to read header data from '$vs->{base}.hdr': $!");
+    $vs->loadHeaderData($hdr)
+      or $vs->logconess("failed to load header data from '$vs->{base}.hdr': $!");
+  }
+
+  ##-- check compatibility
+  my $min_version = qv(0.12.000);
+  if ($hdr && (!defined($hdr->{version}) || version->parse($hdr->{version}) < $min_version)) {
+    $vs->vlog($vs->{logCompat}, "using v0.11 compatibility mode for $vs->{base}.*; consider running \`dcdb-upgrade.perl ", dirname($vs->{base}), "\'");
+    #DiaColloDB::Compat->usecompat('v0_11');
+    DiaColloDB::Compat->usecompat('v0_11::Relation::TDF');
+    bless($vs, 'DiaColloDB::Compat::v0_11::Relation::TDF');
+    $vs->{version} = $hdr->{version};
+    return $vs->open($base,$flags);
   }
 
   ##-- open: maybe create directory
@@ -207,13 +230,15 @@ sub open {
   }
 
   ##-- open: model data
-  my %ioopts = (ReadOnly=>!fcwrite($flags), mmap=>1, log=>$vs->{logIO});
+  my %ioopts = (ReadOnly=>!fcwrite($flags), mmap=>1, log=>$vs->{logio});
   defined($vs->{tdm} = readPdlFile("$vsdir/tdm", class=>'PDL::CCS::Nd', %ioopts))
     or $vs->logconfess("open(): failed to load term-document frequency matrix from $vsdir/tdm.*: $!");
   defined($vs->{tym} = readPdlFile("$vsdir/tym", class=>'PDL::CCS::Nd', %ioopts))
     or $vs->logconfess("open(): failed to load term-year frequency matrix from $vsdir/tym.*: $!");
   defined($vs->{cf}  = readPdlFile("$vsdir/cf.pdl", %ioopts))
     or $vs->logconfess("open(): failed to load cat-frequencies from $vsdir/cf.pdl: $!");
+  defined($vs->{yf}  = readPdlFile("$vsdir/yf.pdl", %ioopts))
+    or $vs->logconfess("open(): failed to load year-frequencies from $vsdir/yf.pdl: $!");
 
   defined(my $ptr0 = $vs->{ptr0} = readPdlFile("$vsdir/tdm.ptr0.pdl", %ioopts))
     or $vs->logwarn("open(): failed to load Harwell-Boeing pointer from $vsdir/tdm.ptr0.pdl: $!");
@@ -248,7 +273,7 @@ sub close {
 #   $vs->{dcmap}->saveDir("$vs->{base}_map.d", %{$vs->{dcio}//{}})
 #     or $vs->logconfess("close(): failed to save mapper data to $vs->{base}_map.d: $!");
   }
-  delete @$vs{qw(base N tdm tw attrs tvals tsorti tpos meta mvals msorti mpos tdm tw)};
+  delete @$vs{qw(base N tdm tym cf yf tw attrs tvals tsorti tpos meta mvals msorti mpos d2c c2d d2date)};
   return $vs;
 }
 
@@ -617,15 +642,25 @@ sub create {
   ##-- create: cf: ($NC): [$ci] -> f($ci)
   $vs->vlog($logCreate, "$logas: creating cat-frequency piddle $vsdir/cf.pdl (NC=$NC)");
   $tdm->_nzvals->indadd( $d2c->index($tdm->_whichND->slice("(1),")), my $cf=mmzeroes("$vsdir/cf.pdl",$vtype,$NC));
+  #undef $cf;
+
+  ##-- create: yf: ($NY): [$yi] -> f($yi)
+  defined(my $c2date = readPdlFile("$c2datefile"))
+    or $vs->logconfess("$logas: failed to mmap $c2datefile");
+  my ($ymin,$ymax) = $c2date->minmax;
+  my $NY = $ymax-$ymin+1;
+  $vs->vlog($logCreate, "$logas: creating year-frequency piddle $vsdir/yf.pdl (NY=$NY)");
+  $cf->indadd( ($c2date-$ymin), my $yf=mmzeroes("$vsdir/yf.pdl",$vtype,$NY) );
+  $vs->{y0} = $ymin;
+
+  ##-- cleanup: yf,cf
+  undef $yf;
   undef $cf;
 
   ##-- create: tym: ($NT,$NY): [$ti,$yi] -> f($ti,$yi)
   $vs->vlog($logCreate, "$logas: creating term-year matrix $vsdir/tym.*");
-  defined(my $c2date = readPdlFile("$c2datefile"))
-    or $vs->logconfess("$logas: failed to mmap $c2datefile");
 
   ##-- tym: create using local memory-optimized pdl-pp method
-  my $ymax = $c2date->max;
   my $tymsub = PDL->can('diacollo_tym_create_'.$vs->itype) || \&PDL::diacollo_tym_create_long;
   $tymsub->($tdm->_whichND, $tdm->_vals, $d2c, $c2date, (my $tym_nnz=pdl($itype,0)), "$vsdir/tym.ix", "$vsdir/tym.nz");
   writePdlHeader("$vsdir/tym.ix.hdr", $itype, 2, 2,$tym_nnz)
@@ -885,7 +920,11 @@ sub export {
   ##-- export: PDLs: dense (mm format)
   require 'PDL/CCS/IO/MatrixMarket.pm'
     or $vs->logconfess("export(): failed to load PDL::CCS::IO::MatrixMarket");
-  foreach my $key (qw(tvals tsorti mvals msorti cf c2date c2d d2c)) {
+  foreach my $key (qw(tvals tsorti mvals msorti cf c2date c2d d2c yf)) {
+    if (!defined($vs->{$key})) {
+      $vs->logwarn("cannot export $outdir/$key.mm : object property {$key} is undefined!");
+      next;
+    }
     $vs->vlog($logLocal,"exporting $outdir/$key.mm");
     $vs->{$key}->writemm("$outdir/$key.mm")
       or $vs->logconfess("export() failed for $outdir/$key.mm: $!");
@@ -914,6 +953,7 @@ sub dbinfo {
   $info->{nTerms} = $vs->nTerms;
   $info->{nDocs}  = $vs->nDocs;
   $info->{nCats}  = $vs->nCats;
+  $info->{nDates} = $vs->nDates;
   return $info;
 }
 
@@ -1156,7 +1196,7 @@ sub vprofile {
 
   ##-- convert packed to native-style profiles (by date-slice)
   my @slices = $sliceby ? (map {$sliceby*$_} (($dlo/$sliceby)..($dhi/$sliceby))) : qw(0);
-  my %dprfs  = map {($_=>DiaColloDB::Profile->new(label=>$_, titles=>$groupby->{titles}, N=>$vs->{N}, f1=>($f1p->{pack($pack_ix,$_)}//0)))} @slices;
+  my %dprfs  = map {($_=>DiaColloDB::Profile->new(label=>$_, titles=>$groupby->{titles}, N=>$vs->sliceN($sliceby,$_), f1=>($f1p->{pack($pack_ix,$_)}//0)))} @slices;
   if (@slices > 1) {
     $vs->vlog($logDebug, "vprofile(): partionining profile data into ", scalar(@slices), " slice(s)");
     my $len_gkey = packsize($pack_gkey);
@@ -1358,6 +1398,12 @@ sub nFiles {
   return $_[0]{c2date}->nelem;
 }
 
+## $NY = $vs->nDates()
+BEGIN { *nYears = \&nDates; }
+sub nDates {
+  return $_[0]{yf}->nelem;
+}
+
 ## $NA = $vs->nAttrs()
 ##  + returns number of term-attributes
 sub nAttrs {
@@ -1480,6 +1526,22 @@ sub catSubset {
 
 
 ##----------------------------------------------------------------------
+## Profile Utils: slice frequency
+
+## $N = $vs->sliceN($sliceBy, $dateLo)
+##  + get total slice co-occurrence count, used by vprofile()
+sub sliceN {
+  my ($vs,$sliceby,$dlo) = @_;
+  return $vs->{N} if ($sliceby==0);
+  my $ymin = $vs->{y0};
+  my $ihi  = min2( $dlo-$ymin+$sliceby, $vs->nDates );
+  my $ilo  = max2( $dlo-$ymin,          0 );
+  return $vs->{yf}->slice("$ilo:".($ihi-1))->sum;
+}
+
+
+##----------------------------------------------------------------------
+## Profile Utils: groupby
 
 ## \%groupby = $vs->groupby($coldb, $groupby_request, %opts)
 ## \%groupby = $vs->groupby($coldb, \%groupby,        %opts)
